@@ -3,7 +3,7 @@ from copy import deepcopy
 
 from .algorithm import Algorithm
 from .operators import RandomGenerator, SimulatedBinaryCrossover, \
-    PmMutator, TournamentSelector, EpsilonDominance
+    PmMutator, TournamentSelector, EpsilonDominance, nondominated_truncate
 from .population import Population
 from .problem import Problem
 
@@ -55,7 +55,7 @@ class GeneticAlgorithm(GeneralEvolutionaryAlgorithm):
 
         self.options.declare(name='n_iterations', default=50, lower=1,
                              desc='Maximum evaluations')
-        self.options.declare(name='max_population_number', default=10, lower=1,
+        self.options.declare(name='max_population_number', default=100, lower=1,
                              desc='max_population_number')
         self.options.declare(name='max_population_size', default=100, lower=1,
                              desc='Maximal number of individuals in population')
@@ -66,20 +66,19 @@ class GeneticAlgorithm(GeneralEvolutionaryAlgorithm):
             individual.features = self.features.copy()
 
     def generate(self, parents, archive=None):
-        """ generate two children from two different parents """
-
-        # Preserve bounds of Pareto-Front
-        children = []
-        while len(children) < self.population_size:
+        offsprings = []
+        while len(offsprings) < self.population_size:
             parent1 = self.selector.select(parents)
 
-            if archive is not None:
-                parent2 = self.selector.select(archive)
-            else:
-                parent2 = self.selector.select(parents)
+            repeat = True
+            while repeat:
+                if archive:
+                    parent2 = self.selector.select(archive)
+                else:
+                    parent2 = self.selector.select(parents)
 
-            while parent1 == parent2:
-                parent2 = self.selector.select(parents)
+                if parent1 is not parent2:
+                    repeat = False
 
             # crossover
             child1, child2 = self.crossover.cross(parent1, parent2)
@@ -87,13 +86,14 @@ class GeneticAlgorithm(GeneralEvolutionaryAlgorithm):
             # mutation
             child1 = self.mutator.mutate(child1)
             child2 = self.mutator.mutate(child2)
-            if not any(child1 == item for item in children):
-                children.append(deepcopy(child1))  # Always create new individual
-            if not any(child1 == item for item in children):
-                children.append(deepcopy(child2))  # Always create new individual
+
+            if not any(child1 == item for item in offsprings):
+                offsprings.append(deepcopy(child1))  # Always create new individual
+            if not any(child1 == item for item in offsprings):
+                offsprings.append(deepcopy(child2))  # Always create new individual
 
         # parents.extend(removed_parents)
-        return children
+        return offsprings
 
     def run(self):
         pass
@@ -102,39 +102,41 @@ class GeneticAlgorithm(GeneralEvolutionaryAlgorithm):
 class NSGAII(GeneticAlgorithm):
 
     def __init__(self, problem: Problem, name="NSGA_II Evolutionary Algorithm"):
+        """
+         NSGA-II implementation as described in
+
+         [1] K. Deb, A. Pratap, S. Agarwal and T. Meyarivan, "A fast and elitist
+             multiobjective genetic algorithm: NSGA-II," in IEEE Transactions on Evolutionary Computation,
+             vol. 6, no. 2, pp. 182-197, Apr 2002. doi: 10.1109/4235.996017
+        """
         super().__init__(problem, name)
 
-        self.options.declare(name='prob_cross', default=0.6, lower=0,
+        self.options.declare(name='prob_cross', default=1.0, lower=0,
                              desc='prob_cross')
-        self.options.declare(name='prob_mutation', default=0.2, lower=0,
+
+        self.options.declare(name='prob_mutation', default=1.0 / (len(problem.parameters)), lower=0,
                              desc='prob_mutation')
+
         self.features = {'dominate': set(),
                          'crowding_distance': 0,
                          'domination_counter': 0,
-                         'front_number': 0}
+                         'front_number': 0, }
 
-    def run(self):
-        # set random generator
         self.generator = RandomGenerator(self.problem.parameters)
-        self.generator.init(self.options['max_population_size'])
-
-        # set crossover
         self.crossover = SimulatedBinaryCrossover(self.problem.parameters, self.options['prob_cross'])
-
-        # set mutator
         self.mutator = PmMutator(self.problem.parameters, self.options['prob_mutation'])
-
-        # set selector
         self.selector = TournamentSelector(self.problem.parameters)
 
+    def run(self):
+        self.generator.init(self.options['max_population_size'])
         # create initial population and evaluate individuals
         population = self.gen_initial_population()
         self.evaluate(population.individuals)
         self.add_features(population.individuals)
 
         # non-dominated sort of individuals
-        self.selector.sorting(population.individuals)
-        self.selector.crowding_distance(population.individuals)
+        self.selector.fast_nondominated_sorting(population.individuals)
+        # self.selector.crowding_distance(population.individuals)
 
         t_s = time.time()
         self.problem.logger.info(
@@ -154,15 +156,10 @@ class NSGAII(GeneticAlgorithm):
             # add the parents to the offsprings
             offsprings.extend(deepcopy(population.individuals))
 
-            # non-dominated truncate on the individuals
-            self.selector.sorting(offsprings)
-            self.selector.crowding_distance(offsprings)
-
-            # sort offsprings and ### remove duplicates by set!!!
-            parents = sorted(set(offsprings), key=lambda x: (x.features['front_number'], -x.features['crowding_distance']))
-
-            # truncate and replace individuals
-            population.individuals = parents[:self.population_size]
+            # make the pareto dominance calculation and calculating the crowding distance
+            self.selector.fast_nondominated_sorting(offsprings)
+            # self.selector.crowding_distance(offsprings)
+            population.individuals = nondominated_truncate(offsprings, self.population_size)
 
         t = time.time() - t_s
         self.problem.logger.info("NSGA_II: elapsed time: {} s".format(t))
@@ -185,7 +182,6 @@ class EpsMOEA(GeneticAlgorithm):
                          'domination_counter': 0,
                          'front_number': 0}
 
-
     def run(self):
         # set random generator
         self.generator = RandomGenerator(self.problem.parameters)  # the same as in the case of NSGA-II
@@ -206,14 +202,14 @@ class EpsMOEA(GeneticAlgorithm):
         population = self.gen_initial_population(True)  # archiving True
         self.evaluate(population.individuals)
         self.add_features(population.individuals)
-        selector_pareto.sorting(population.individuals)
+        selector_pareto.fast_nondominated_sorting(population.individuals)
         selector_pareto.crowding_distance(population.individuals)
 
         # Part B: eps-dominated sort of the individuals with archiving
         # -----
         selector_epsdom = TournamentSelector(self.problem.parameters,
                                              dominance=EpsilonDominance, epsilons=self.options['epsilons'])
-        selector_epsdom.sorting(population.archives)
+        selector_epsdom.fast_nondominated_sorting(population.archives)
         selector_epsdom.crowding_distance(population.archives)
 
         t_s = time.time()
@@ -237,17 +233,19 @@ class EpsMOEA(GeneticAlgorithm):
             children.extend(deepcopy(population.individuals))
 
             # non-dominated truncate on the guys
-            self.selector.sorting(children)
+            self.selector.fast_nondominated_sorting(children)
             selector_pareto.crowding_distance(children)
 
-            parents = sorted(set(children), key=lambda x: (x.features['front_number'], -x.features['crowding_distance']))
+            parents = sorted(set(children),
+                             key=lambda x: (x.features['front_number'], -x.features['crowding_distance']))
             child = parents[:self.population_size]  # truncate
 
             # eps dominated truncate on the guys
-            selector_epsdom.sorting(arch_child)
+            selector_epsdom.fast_nondominated_sorting(arch_child)
             selector_epsdom.crowding_distance(arch_child)
 
-            arch_parents = sorted(set(arch_child), key=lambda x: (x.features['front_number'], -x.features['crowding_distance']))
+            arch_parents = sorted(set(arch_child),
+                                  key=lambda x: (x.features['front_number'], -x.features['crowding_distance']))
             population.archives = arch_parents[:self.population_size]  # truncate
 
         t = time.time() - t_s
