@@ -5,11 +5,13 @@ import math
 import itertools
 from math import exp
 import numpy as np
+import functools
 from artap.individual import Individual
 from artap.utils import VectorAndNumbers
 from artap.doe import build_box_behnken, build_lhs, build_full_fact, build_plackett_burman
-from .job import Job
+from artap.job import Job
 from joblib import Parallel, delayed
+from copy import deepcopy
 
 EPSILON = sys.float_info.epsilon
 
@@ -46,10 +48,11 @@ class Evaluator(Operator):
             self.evaluate_serial(individuals)
 
         n_failed = 0
-        for individual in individuals:
-            if individual.state == Individual.State.FAILED:
-                n_failed += 1
-                individuals.remove(individual)   # TODO: is can be not feasible?
+        # for individual in individuals:
+        #     # if individual.state == Individual.State.FAILED:
+        #     #     n_failed += 1
+        #     #     individuals.remove(individual)   # TODO: it is not correct, nsga2 handles this case in paretodominance
+        #     individual.calc_signed_costs(self.problem.signs) # the idea is to make this conversion only once
 
     def evaluate_serial(self, individuals: list):
         job = Job(self.algorithm.problem)
@@ -60,8 +63,10 @@ class Evaluator(Operator):
     def evaluate_parallel(self, individuals: list):
         # simple parallel loop
         job = Job(self.algorithm.problem)
-        Parallel(n_jobs=self.algorithm.options["max_processes"], verbose=1, require='sharedmem')(delayed(job.evaluate)(individual)
-                                                                                       for individual in individuals)
+        Parallel(n_jobs=self.algorithm.options["max_processes"], verbose=1, require='sharedmem')(
+            delayed(job.evaluate)(individual)
+            for individual in individuals)
+
 
 class GradientEvaluator(Evaluator):
 
@@ -335,10 +340,7 @@ class PmMutator(Mutator):
     PmMutation -- for nsga2 and epsMoEA
 
     This operator can handle real, integer and boolean optimization parameters.
-    The class contains two-kind of operators as the original (Deb's [...]) implementation .
-
-    This is a difference between Artap and Platypus, where the Integer numbers are encoded (Gray-encoding) and handled
-    as binary numbers with the Bitflip operator.
+    The class contains two-kind of operators as the original Deb's implementation .
     """
 
     def __init__(self, parameters, probability, distribution_index=20):
@@ -349,24 +351,14 @@ class PmMutator(Mutator):
         vector = []
 
         for i, parameter in enumerate(self.parameters):
-            if isinstance(parent.vector[i], list):
-                vector.append(self.bitflip(parent.vector[i]))
-
+            if random.uniform(0, 1) < self.probability:
+                l_b = parameter['bounds'][0]
+                u_b = parameter['bounds'][1]
+                vector.append(self.pm_mutation(parent.vector[i], l_b, u_b))
             else:
-                if random.uniform(0, 1) < self.probability:
-                    l_b = parameter['bounds'][0]
-                    u_b = parameter['bounds'][1]
+                vector.append(parent.vector[i])
 
-                    if isinstance(parent.vector[i], float):
-                        vector.append(self.pm_mutation(parent.vector[i], l_b, u_b))
-
-                    if isinstance(parent.vector[i], int):
-                        vector.append(int(self.pm_mutation(parent.vector[i], l_b, u_b)))
-                else:
-                    vector.append(parent.vector[i])
-
-        p_new = parent.__class__(vector)
-        return p_new
+        return parent.__class__(vector)
 
     def pm_mutation(self, x, lb, ub):
         """
@@ -377,19 +369,21 @@ class PmMutator(Mutator):
         :param ub: upper bound
         :return:
         """
-        u = random.uniform(0, 1)
+        rnd = random.uniform(0, 1)
         dx = ub - lb
-
-        if u < 0.5:
-            bl = (x - lb) / dx
-            b = 2.0 * u + (1.0 - 2.0 * u) * pow(1.0 - bl, self.distribution_index + 1.0)
-            delta = pow(b, 1.0 / (self.distribution_index + 1.0)) - 1.0
+        delta1 = (x - lb) / dx
+        delta2 = (ub - x) / dx
+        mut_pow = 1.0 / (self.distribution_index + 1.0)
+        if rnd < 0.5:
+            xy = 1.0 - delta1
+            val = 2.0 * rnd + (1.0 - 2.0 * rnd) * (pow(xy, self.distribution_index + 1.0))
+            deltaq = pow(val, mut_pow) - 1.0
         else:
-            bu = (ub - x) / dx
-            b = 2.0 * (1.0 - u) + 2.0 * (u - 0.5) * pow(1.0 - bu, self.distribution_index + 1.0)
-            delta = 1.0 - pow(b, 1.0 / (self.distribution_index + 1.0))
+            xy = 1.0 - delta2
+            val = 2.0 * (1.0 - rnd) + 2.0 * (rnd - 0.5) * (pow(xy, self.distribution_index + 1.0))
+            deltaq = 1.0 - pow(val, mut_pow)
 
-        x = x + delta * dx
+        x = x + deltaq * dx
         x = self.clip(x, lb, ub)
 
         return x
@@ -438,15 +432,16 @@ class SwarmMutator(Mutator):
 
     # update new particle velocity
     def update_velocity(self, individual):
-        individual.features['velocity'] = [0]*len(individual.vector)
-        individual.features['best_vector'] = [0]*len(individual.vector)
+        individual.features['velocity'] = [0] * len(individual.vector)
+        individual.features['best_vector'] = [0] * len(individual.vector)
         for i in range(0, len(individual.vector)):
             r1 = 0.1 * random.random()
             r2 = 0.1 * random.random()
 
             vel_cognitive = self.c1 * r1 * (individual.features['best_vector'][i] - individual.vector[i])
             vel_social = self.c2 * r2 * (self.best_individual.vector[i] - individual.vector[i])
-            individual.features['velocity'][i] = self.w * individual.features['velocity'][i] + vel_cognitive + vel_social
+            individual.features['velocity'][i] = self.w * individual.features['velocity'][
+                i] + vel_cognitive + vel_social
 
     # update the particle position based off new velocity updates
     def update_position(self, individual):
@@ -513,8 +508,8 @@ class SwarmMutatorTVIW(SwarmMutator):
         :param iteration_nr: actual generation
         """
 
-        individual.features['velocity'] = [0]*len(individual.vector)
-        individual.features['best_vector'] = [0]*len(individual.vector)
+        individual.features['velocity'] = [0] * len(individual.vector)
+        individual.features['best_vector'] = [0] * len(individual.vector)
         for i in range(0, len(individual.vector)):
             r1 = 0.1 * random.random()
             r2 = 0.1 * random.random()
@@ -618,7 +613,7 @@ class FireflyMutator(SwarmMutator):
         dominates = True
 
         for i in range(len(current.costs)):
-            if other.signed_costs[i] > current.signed_costs[i]:
+            if other.signs[i] > current.signs[i]:
                 dominates = False
 
         return dominates
@@ -637,7 +632,6 @@ class FireflyMutator(SwarmMutator):
         r2 = 0.  # euclidean distance
 
         for i, param in enumerate(self.parameters):
-
             lb = param['bounds'][0]
             ub = param['bounds'][1]
 
@@ -650,45 +644,6 @@ class FireflyMutator(SwarmMutator):
             v_rd = self.alpha * (random.random() - 0.5) * e
             current.velocity_i[i] = vel_attraction + v_rd
 
-            # for individual in offsprings:
-            #     for other in offsprings:
-            #         dominates = False
-            #
-            #         if individual is not other:
-            #             dominates = True
-            #
-            #         for i in range(len(individual.signed_costs)):
-            #             if individual.signed_costs[i] > offsprings.signed_costs[i]:
-            #                 dominates = False
-            #
-            #         if dominates:
-            #
-            #             r2 = 0
-            #             for i, param in enumerate(self.parameters):
-            #                 l_b = param['bounds'][0]
-            #                 u_b = param['bounds'][1]
-            #
-            #                 # elementary distance of the particle (to do not )
-            #                 e = self.probability*(u_b-l_b)
-            #                 # distance between the two individuals
-            #                 r2 += individual.vector[i]**2. + other.vector[i]**2.
-            #
-            #                 vel_attraction = self.beta*exp(-self.gamma*r2**2.)
-            #
-            #                 v_rd = self.alpha * (random.random() - 0.5) * e
-            #                 individual.velocity_i[i] = vel_attraction + v_rd
-            #
-            #                 temp = copy(individual)
-            #                 self.evaluate(temp)
-            #
-            #                 dom = True
-            #                 for j in range(len(individual.best_costs)):
-            #                     if individual.signed_costs[j] > offsprings.signed_costs[j]:
-            #                         dom = False
-            #
-            #                 if dom is True:
-            #                     individual = copy(temp)
-
             return
 
     def mutate_ij(self, p, q):
@@ -699,7 +654,7 @@ class FireflyMutator(SwarmMutator):
         :param q:
         :return:
         """
-        self.update_velocity_ij(p,q)
+        self.update_velocity_ij(p, q)
         self.update_position(p)
         return
 
@@ -751,111 +706,6 @@ class SwarmMutatorTVAC(SwarmMutator):
             individual.velocity_i[i] = self.w * individual.velocity_i[i] + vel_cognitive + vel_social
 
 
-class Selector(Operator):
-
-    def __init__(self, parameters, part_num=2):
-        super().__init__()
-        self.parameters = parameters
-        # self.signs = signs
-        self.part_num = part_num
-        self.comparator = ParetoDominance()
-
-    @abstractmethod
-    def select(self, population):
-        pass
-
-    def is_dominate(self, p, q):
-        """
-        :param p: current solution
-        :param q: candidate
-        :return: True if the candidate is better than the current solution
-        """
-        dominate = False
-
-        # The cost function can be a float or a list of floats
-        for i in range(0, len(p.costs)):
-            if p.costs[i] > q.costs[i]:
-                return False
-            if p.costs[i] <= q.costs[i]:
-                dominate = True
-        return dominate
-
-    def non_dominated_sort(self, population):
-        pareto_front = []
-        front_number = 1
-
-        for p in population:
-            p.domination_counter = 0
-            p.front_number = None
-            p.dominate = set()
-
-            for q in population:
-                if p is q:
-                    continue
-                if self.comparator.compare(p, q) == 1:  # TODO: simplify
-                    p.dominate.add(q)
-                elif self.comparator.compare(q, p) == 1:
-                    p.domination_counter = p.domination_counter + 1
-
-            if p.domination_counter == 0:
-                p.front_number = front_number
-                pareto_front.append(p)
-
-        while not len(pareto_front) == 0:
-            front_number += 1
-            temp_set = []
-            for p in pareto_front:
-                for q in p.dominate:
-                    q.domination_counter -= 1
-                    if q.domination_counter == 0 and q.front_number is None:
-                        q.front_number = front_number
-                        temp_set.append(q)
-            pareto_front = temp_set
-
-    def sort_by_coordinate(self, population, dim):
-        population.sort(key=lambda x: x.costs[dim])
-        return population
-
-    def crowding_distance(self, population):
-        infinite = float("inf")
-        n = len(population[0].costs)
-        for dim in range(0, n):
-            new_list = self.sort_by_coordinate(population, dim)
-
-            new_list[0].crowding_distance += infinite
-            new_list[-1].crowding_distance += infinite
-            max_distance = new_list[0].vector[dim] - new_list[-1].vector[dim]
-            for i in range(1, len(new_list) - 1):
-                distance = new_list[i - 1].vector[dim] - new_list[i + 1].vector[dim]
-                if max_distance == 0:
-                    new_list[i].crowding_distance = 0
-                else:
-                    new_list[i].crowding_distance += distance / max_distance
-
-        for p in population:
-            p.crowding_distance = p.crowding_distance / n
-
-
-class DummySelector(Selector):
-
-
-    def __init__(self, parameters, part_num=2):
-        super().__init__(parameters, part_num)
-
-    def select(self, individuals):
-        selection = []
-        for individual in individuals:
-            candidate = individual.__class__(individual.vector)
-            candidate.costs = individual.costs
-            candidate.front_number = individual.front_number
-            candidate.features['best_vector'] = individual.features['best_vector']
-            candidate.features['best_costs'] = individual.features['best_costs']
-
-            selection.append(candidate)
-
-        return selection
-
-
 class Dominance(ABC):
     def __init__(self):
         pass
@@ -930,19 +780,19 @@ class EpsilonDominance(Dominance):
         # it means that the solution is feasible
         if p[-1] != q[-1]:
             if p[-1] == 0:
-                return 1 # p dominates
+                return 1  # p dominates
             elif q[-1] == 0:
-                return 2 # q is dominates
+                return 2  # q is dominates
             elif p[-1] < q[-1]:
-                return 1 # p is dominates
+                return 1  # p is dominates
             elif q[-1] < p[-1]:
-                return 2 # q is dominates
+                return 2  # q is dominates
 
         # then use epsilon dominance on the objectives
         dominate_p = False
         dominate_q = False
 
-        for i,(p_costs, q_costs) in enumerate(p[:-1], q[:-1]):
+        for i, (p_costs, q_costs) in enumerate(p[:-1], q[:-1]):
 
             epsilon = float(self.epsilons[i % len(self.epsilons)])
 
@@ -953,12 +803,11 @@ class EpsilonDominance(Dominance):
                 dominate_q = True
                 if dominate_p:
                     return 0
-            else:
+            elif qepsi > pepsi:
                 dominate_p = True
                 if dominate_q:
                     return 0
 
-
         if not dominate_p and not dominate_q:
             dist1 = 0.0
             dist2 = 0.0
@@ -988,7 +837,7 @@ class ParetoDominance(Dominance):
     """Pareto dominance with constraints.
 
     If either solution violates constraints, then the solution with a smaller
-    constraint violation is preferred.  If both solutions are feasible, then
+    constraint violation is preferred. If both solutions are feasible, then
     Pareto dominance is used to select the preferred solution.
     """
 
@@ -1001,10 +850,22 @@ class ParetoDominance(Dominance):
         Here, p and q are tuples, which contains the (feasibility index, cost vector)
         """
 
+        # first check constraint violation, the last item is the feasibility, which is a real number if its zero,
+        # it means that the solution is feasible
+        if p[-1] != q[-1]:
+            if p[-1] == 0:
+                return 1  # p dominates
+            elif q[-1] == 0:
+                return 2  # q is dominates
+            elif p[-1] < q[-1]:
+                return 1  # p is dominates
+            elif q[-1] < p[-1]:
+                return 2  # q is dominates
+
         dominate_p = False
         dominate_q = False
 
-        for (p_costs, q_costs) in zip(p, q):
+        for (p_costs, q_costs) in zip(p[:-1], q[:-1]):
 
             if p_costs > q_costs:
                 dominate_q = True
@@ -1035,7 +896,6 @@ class Selector(Operator):
         """
         super().__init__()
         self.parameters = parameters
-        self.part_num = part_num
         self.comparator = dominance()  # ParetoDominance()
         self.signs = sign
 
@@ -1043,79 +903,134 @@ class Selector(Operator):
     def select(self, population):
         pass
 
-    def is_dominate(self, p, q):
-        """
-        :param p: current solution
-        :param q: candidate
-        :return: True if the candidate is better than the current solution
-        """
-        # The cost function can be a float or a list of floats
-        for p_costs, q_costs in zip(p.costs, q.costs):
-            if p_costs > q_costs:
-                return False
-            else:
-                return True
+    def fast_nondominated_sorting(self, population):
 
-    def sorting(self, generation):
-        """
-        This shoring can be dominated or non-dominated.
-        :param generation: means the list of the population, it can be list of the individuals or the archive.
-        :return:
-        """
-        pareto_front = []
+        pareto_front = [[]]
         front_number = 1
 
-        for p in generation:
+        # reset elements
+        for p in population:
             p.features['domination_counter'] = 0
             p.features['front_number'] = None
             p.features['dominate'] = set()
 
-            for q in generation:
-                if p is q:
-                    continue
-                if self.comparator.compare(p.signed_costs(), q.signed_costs()) == 1:
+        for i, p in enumerate(population):
+            # for q in population:
+            for j in range(i + 1, len(population)):
+                q = population[j]
+                # if p is q:
+                #    continue
+                dom = self.comparator.compare(p.signs, q.signs)
+                if dom == 1:
                     p.features['dominate'].add(q)
-                elif self.comparator.compare(q.signed_costs(), p.signed_costs()) == 1:
+                    q.features['domination_counter'] += 1
+                elif dom == 2:
                     p.features['domination_counter'] += 1
+                    q.features['dominate'].add(p)
 
+            # selects the pareto values
             if p.features['domination_counter'] == 0:
                 p.features['front_number'] = front_number
-                pareto_front.append(p)
+                pareto_front[front_number - 1].append(p)
 
-        while not len(pareto_front) == 0:
+        while len(pareto_front[front_number - 1]) > 0:
             front_number += 1
-            temp_set = []
-            for p in pareto_front:
+            pareto_front.append([])
+            for p in pareto_front[front_number - 2]:
                 for q in p.features['dominate']:
                     q.features['domination_counter'] -= 1
                     if q.features['domination_counter'] == 0 and q.features['front_number'] is None:
                         q.features['front_number'] = front_number
-                        temp_set.append(q)
-            pareto_front = temp_set
+                        pareto_front[front_number - 1].append(q)
+                        # temp_set.append(q)
+            # pareto_front = temp_set
+        pareto_front.pop()
+        for sub_front in pareto_front:
+            crowding_distance(sub_front)
+
         return
 
-    def sort_by_coordinate(self, population, dim):
-        population.sort(key=lambda x: x.costs[dim])
-        return population
 
-    def crowding_distance(self, population):
-        infinite = float("inf")
-        n = len(population[0].costs)
-        for dim in range(0, n):
-            new_list = self.sort_by_coordinate(population, dim)
+def crowding_distance(front):
+    """
+    Crowding distance calculates the solution density on a front, a subset of the population.
+    :param front: list of individuals
+                  which is a subset of the total population
+    :return:
+    """
+    n = len(front)
 
-            new_list[0].features['crowding_distance'] += infinite
-            new_list[-1].features['crowding_distance'] += infinite
-            max_distance = new_list[0].vector[dim] - new_list[-1].vector[dim]
-            for i in range(1, len(new_list) - 1):
-                distance = new_list[i - 1].vector[dim] - new_list[i + 1].vector[dim]
-                if max_distance == 0:
-                    new_list[i].features['crowding_distance'] = 0
-                else:
-                    new_list[i].features['crowding_distance'] += distance / max_distance
+    if n is 0:
+        return
+    elif n is 1:
+        front[0].features['crowding_distance'] = math.inf
+        return
+    elif n is 2:
+        front[0].features['crowding_distance'] = math.inf
+        front[1].features['crowding_distance'] = math.inf
+        return
 
-        for p in population:
-            p.features['crowding_distance'] = p.features['crowding_distance'] / n
+    for i in range(len(front)):
+        front[i].features['crowding_distance'] = 0.0
+
+    for dim in range(len(front[0].signs[:-1])):
+
+        front.sort(key=lambda x: x.signs[dim])
+        # self.sort_by_coordinate(population, dim)
+
+        front[0].features['crowding_distance'] = math.inf
+        front[-1].features['crowding_distance'] = math.inf
+        max_distance = front[-1].signs[dim] - front[0].signs[dim]
+        for i in range(1, n - 1):
+            distance = front[i + 1].signs[dim] - front[i - 1].signs[dim]
+            if max_distance > 0.0:
+                front[i].features['crowding_distance'] += distance / max_distance
+    return
+
+
+def nondominated_truncate(population, size):
+    """Truncates a population to the given size, using non-dominated sorting.
+    The resulting population is filled with the first N-1 fronts.
+    The Nth front is too large and must be split using crowding distance.
+
+    Parameters
+    ----------
+    :population : iterable
+        The collection of solutions that have been non-domination sorted
+    :size:
+        The size of the truncated result
+    """
+
+    # calculating the crowding distance on the different fronts
+
+    result = sorted(population, key=functools.cmp_to_key(nondominated_cmp))
+    return result[:size]
+
+
+def nondominated_cmp(p, q):
+    """
+    From the 'front_number' the smaller value is favourized.
+    From the 'crowding_distance' the higher values are favoured.
+    :param p: Individual
+    :param q: Individual
+    :return: -1 if x dominates
+              0 if they are equals
+              1 if y dominates
+    """
+    if p.features['front_number'] == q.features['front_number']:
+        if -p.features['crowding_distance'] < -q.features['crowding_distance']:
+            return -1
+        elif -p.features['crowding_distance'] > -q.features['crowding_distance']:
+            return 1
+        else:
+            return 0
+    else:
+        if p.features['front_number'] < q.features['front_number']:
+            return -1
+        elif p.features['front_number'] > q.features['front_number']:
+            return 1
+        else:
+            return 0
 
 
 class DummySelector(Selector):
@@ -1138,32 +1053,32 @@ class TournamentSelector(Selector):
     def __init__(self, parameters, dominance=ParetoDominance, epsilons=None):
         super().__init__(parameters)
         self.dominance = dominance(epsilons=epsilons)
-        # self.signs = sign
-        # self.dominance = EpsilonDominance([0.01])
 
     def select(self, population):
         """
         Binary tournament selection:
-        An individual is selected in the rank is lesser than the other or if crowding distance is greater than the other
+
+        2 individuals are selected from the given population (randomly), then the function gaves
+        back the dominant (or random).
         """
 
-        # participants = random.sample(population, 2)
-        # return min(participants, key=lambda x: (x.front_number, -x.crowding_distance))
+        if len(population) == 1:
+            selected = population[0]
 
-        winner = random.choice(population)
+        else:
+            # Sampling two individuals without a replacement
+            candidates = random.sample(population, 2)
 
-        for _ in range(self.part_num - 1):
-            candidate = random.choice(population)
+            flag = self.dominance.compare(candidates[0].signs, candidates[1].signs)
 
-            flag = self.dominance.compare(winner.signed_costs(), candidate.signed_costs())
+            if flag == 1:
+                selected = candidates[0]
+            elif flag == 2:
+                selected = candidates[1]
+            else:
+                selected = random.choice(candidates)
 
-            if flag > 0:
-                winner = candidate
-
-        return winner
-
-        # participants = random.sample(parents, part_num)
-        # return min(participants, key=lambda x: (x.front_number, -x.crowding_distance))
+        return selected
 
 
 class Archive(object):
@@ -1286,90 +1201,70 @@ class SimpleCrossover(Crossover):
 
 class SimulatedBinaryCrossover(Crossover):
 
-    def __init__(self, parameters, probability, distribution_index=15):
+    def __init__(self, parameters, probability, distribution_index=20):
         super().__init__(parameters, probability)
         self.distribution_index = distribution_index
-
-    def sbx(self, x1, x2, lb, ub, p_type="real"):
-
-        dx = x2 - x1
-
-        if dx > EPSILON:
-            if x2 > x1:
-                y2 = x2
-                y1 = x1
-            else:
-                y2 = x1
-                y1 = x2
-
-            beta = 1.0 / (1.0 + (2.0 * (y1 - lb) / (y2 - y1)))
-            alpha = 2.0 - pow(beta, self.distribution_index + 1.0)
-            rand = random.uniform(0.0, 1.0)
-
-            if rand <= 1.0 / alpha:
-                alpha = alpha * rand
-                betaq = pow(alpha, 1.0 / (self.distribution_index + 1.0))
-            else:
-                alpha = alpha * rand
-                alpha = 1.0 / (2.0 - alpha)
-                betaq = pow(alpha, 1.0 / (self.distribution_index + 1.0))
-
-            x1 = 0.5 * ((y1 + y2) - betaq * (y2 - y1))
-            beta = 1.0 / (1.0 + (2.0 * (ub - y2) / (y2 - y1)))
-            alpha = 2.0 - pow(beta, self.distribution_index + 1.0)
-
-            if rand <= 1.0 / alpha:
-                alpha = alpha * rand
-                betaq = pow(alpha, 1.0 / (self.distribution_index + 1.0))
-            else:
-                alpha = alpha * rand
-                alpha = 1.0 / (2.0 - alpha)
-                betaq = pow(alpha, 1.0 / (self.distribution_index + 1.0))
-
-            x2 = 0.5 * ((y1 + y2) + betaq * (y2 - y1))
-
-            # randomly swap the values
-            if bool(random.getrandbits(1)):
-                x1, x2 = x2, x1
-
-            x1 = self.clip(x1, lb, ub)
-            x2 = self.clip(x2, lb, ub)
-
-            if p_type == "integer":
-                x1 = int(x1)
-                x2 = int(x2)
-
-        return x1, x2
+        if distribution_index < 0:
+            raise Exception('The distribution index have to be positive.')
+        if probability > 1.:
+            raise Exception('The probability should be selected from [0,1].')
+        elif probability < 0.:
+            raise Exception('The probability should be selected from [0,1].')
 
     def cross(self, p1, p2):
         """
         Create an offspring using simulated binary crossover.
-        :return:  a list with 2 offsprings each with the genotype of an  offspring after recombination and mutation.
+        :return:  Individual, Individual
+
+        a list with 2 offsprings each with the genotype of an  offspring after recombination and mutation.
         """
 
-        parent_a = p1.vector.copy()
-        parent_b = p2.vector.copy()
+        x1 = deepcopy(p1.vector)
+        x2 = deepcopy(p2.vector)
 
-        if random.uniform(0.0, 1.0) <= self.probability:
+        if random.random() <= self.probability:
             for i, param in enumerate(self.parameters):
-                if random.uniform(0.0, 1.0) <= 0.5:
-                    x1 = parent_a[i]
-                    x2 = parent_b[i]
 
-                    lb = param['bounds'][0]
-                    ub = param['bounds'][1]
+                if random.random() <= 0.5:
 
-                    if not 'parameter_type' in param:
-                        continue
-                    else:
-                        p_type = param['parameter_type']
+                    if abs(x2[i] - x1[i]) > EPSILON:
+                        if x2[i] > x1[i]:
+                            y1, y2 = x1[i], x2[i]
+                        else:
+                            y1, y2 = x2[i], x1[i]
 
-                    x1, x2 = self.sbx(x1, x2, lb, ub, p_type)
+                        lb, ub = param['bounds'][0], param['bounds'][1]
 
-                    parent_a[i] = x1
-                    parent_b[i] = x2
+                        # calculates c1
+                        beta = 1.0 + (2.0 * (y1 - lb) / (y2 - y1))
+                        alpha = 2.0 - pow(beta, -(self.distribution_index + 1.0))
 
-        offspring_a = p1.__class__(parent_a)
-        offspring_b = p2.__class__(parent_b)
+                        rand = random.random()
+                        if rand <= (1.0 / alpha):
+                            betaq = pow(rand * alpha, (1.0 / (self.distribution_index + 1.0)))
+                        else:
+                            betaq = pow(1.0 / (2.0 - rand * alpha), 1.0 / (self.distribution_index + 1.0))
 
-        return offspring_a, offspring_b
+                        c1 = 0.5 * (y1 + y2 - betaq * (y2 - y1))
+
+                        # calculates c2
+                        beta = 1.0 + (2.0 * (ub - y2) / (y2 - y1))
+                        alpha = 2.0 - pow(beta, -(self.distribution_index + 1.0))
+
+                        if rand <= (1.0 / alpha):
+                            betaq = pow((rand * alpha), (1.0 / (self.distribution_index + 1.0)))
+                        else:
+                            betaq = pow(1.0 / (2.0 - rand * alpha), 1.0 / (self.distribution_index + 1.0))
+
+                        c2 = 0.5 * (y1 + y2 + betaq * (y2 - y1))
+
+                        # check the boundaries
+                        c1 = self.clip(c1, lb, ub)
+                        c2 = self.clip(c2, lb, ub)
+
+                        if random.random() <= 0.5:
+                            x1[i], x2[i] = c2, c1
+                        else:
+                            x1[i], x2[i] = c1, c2
+
+        return p1.__class__(x1), p2.__class__(x2)
