@@ -4,26 +4,28 @@ from artap.problem import Problem
 from artap.benchmark_functions import BinhAndKorn, AlpineFunction, Ackley
 from artap.algorithm_genetic import NSGAII
 from artap.results import Results
+#
+#
+# class BinhAndKornProblem(Problem):
+#     """
+#     Describe simple one objective optimization problem.
+#     Note: This test is not removed, because its not part of the cec2005 library.
+#     """
+#
+#     def set(self):
+#         self.name = "TestNSGA2Optimization"
+#         self.parameters = [{'name': 'x_1', 'initial_value': 2.5, 'bounds': [0, 5]},
+#                            {'name': 'x_2', 'initial_value': 1.5, 'bounds': [0, 3]}]
+#         self.costs = [{'name': 'F_1', 'criteria': 'minimize'}, {'name': 'F_2', 'criteria': 'minimize'}]
+#
+#     def evaluate(self, individual):
+#         function = BinhAndKorn()
+#         return function.eval(individual.vector)
+#
+#     def evaluate_constraints(self, individual):
+#         return BinhAndKorn.constraints(individual.vector)
 
 
-class BinhAndKornProblem(Problem):
-    """
-    Describe simple one objective optimization problem.
-    Note: This test is not removed, because its not part of the cec2005 library.
-    """
-
-    def set(self):
-        self.name = "TestNSGA2Optimization"
-        self.parameters = [{'name': 'x_1', 'initial_value': 2.5, 'bounds': [0, 5]},
-                           {'name': 'x_2', 'initial_value': 1.5, 'bounds': [0, 3]}]
-        self.costs = [{'name': 'F_1', 'criteria': 'minimize'}, {'name': 'F_2', 'criteria': 'minimize'}]
-
-    def evaluate(self, individual):
-        function = BinhAndKorn()
-        return function.eval(individual.vector)
-
-    def evaluate_constraints(self, individual):
-        return BinhAndKorn.constraints(individual.vector)
 
 
 class TestNSGA2Optimization(unittest.TestCase):
@@ -82,13 +84,13 @@ class TestNSGA2Optimization(unittest.TestCase):
 class TestAckley(unittest.TestCase):
     """ Tests that the nsga - ii can find the global optimum. """
 
-    def test_local_problem(self, population_number=15):
+    def test_local_problem(self, population_number=50):
         try:
             problem = Ackley(**{'dimension': 1})
             algorithm = NSGAII(problem)
             algorithm.options['max_population_number'] = population_number
             algorithm.options['max_population_size'] = 50
-            algorithm.options['max_processes'] = 10
+            algorithm.options['max_processes'] = 1
             algorithm.run()
 
             b = Results(problem)
@@ -103,13 +105,13 @@ class TestAckley(unittest.TestCase):
 class TestAlpine(unittest.TestCase):
     """ Tests that the nsga - ii can find the global optimum. """
 
-    def test_local_problem(self, population_number=15):
+    def test_local_problem(self, population_number=50):
         try:
             problem = AlpineFunction(**{'dimension': 3})
             algorithm = NSGAII(problem)
             algorithm.options['max_population_number'] = population_number
             algorithm.options['max_population_size'] = 100
-            algorithm.options['max_processes'] = 10
+            algorithm.options['max_processes'] = 1
             algorithm.run()
 
             b = Results(problem)
