@@ -16,7 +16,7 @@ class TestSimpleOptimization(unittest.TestCase):
         algorithm.run()
 
         results = Results(problem)
-        optimum = results.find_minimum('F_1')
+        optimum = results.find_optimum('F_1')
         self.assertAlmostEqual(optimum.costs[0], 0)
 
 
@@ -28,13 +28,12 @@ class TestAckleyN2(unittest.TestCase):
         algorithm = ScipyOpt(problem)
         algorithm.options['algorithm'] = 'Nelder-Mead'
         algorithm.options['tol'] = 1e-4
-        algorithm.options['calculate_gradients'] = True
         algorithm.run()
 
         print(problem.populations[0].individuals[3])
 
         results = Results(problem)
-        optimum = results.find_minimum('F_1')
+        optimum = results.find_optimum('F_1')
         self.assertAlmostEqual(optimum.costs[0], problem.global_optimum, 3)
 
 
